"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
class DebugPlatformCommand {
    constructor(debugService, $devicesService, $debugDataService, $platformService, $projectData, $options, $platformsData, $logger, $errors, $debugLiveSyncService, $config) {
        this.debugService = debugService;
        this.$devicesService = $devicesService;
        this.$debugDataService = $debugDataService;
        this.$platformService = $platformService;
        this.$projectData = $projectData;
        this.$options = $options;
        this.$platformsData = $platformsData;
        this.$logger = $logger;
        this.$errors = $errors;
        this.$debugLiveSyncService = $debugLiveSyncService;
        this.$config = $config;
        this.allowedParameters = [];
        this.$projectData.initializeProjectData();
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const debugOptions = this.$options;
            let debugData = this.$debugDataService.createDebugData(this.$projectData, this.$options);
            yield this.$platformService.trackProjectType(this.$projectData);
            if (this.$options.start) {
                return this.$debugLiveSyncService.printDebugInformation(yield this.debugService.debug(debugData, debugOptions));
            }
            this.$config.debugLivesync = true;
            yield this.$devicesService.initialize({});
            yield this.$devicesService.detectCurrentlyAttachedDevices();
            const devices = this.$devicesService.getDeviceInstances();
            const deviceDescriptors = devices.filter(d => !this.platform || d.deviceInfo.platform === this.platform)
                .map(d => {
                const info = {
                    identifier: d.deviceInfo.identifier,
                    buildAction: () => __awaiter(this, void 0, void 0, function* () {
                        const buildConfig = {
                            buildForDevice: !d.isEmulator,
                            projectDir: this.$options.path,
                            clean: this.$options.clean,
                            teamId: this.$options.teamId,
                            device: this.$options.device,
                            provision: this.$options.provision,
                            release: this.$options.release,
                            keyStoreAlias: this.$options.keyStoreAlias,
                            keyStorePath: this.$options.keyStorePath,
                            keyStoreAliasPassword: this.$options.keyStoreAliasPassword,
                            keyStorePassword: this.$options.keyStorePassword
                        };
                        yield this.$platformService.buildPlatform(d.deviceInfo.platform, buildConfig, this.$projectData);
                        const pathToBuildResult = yield this.$platformService.lastOutputPath(d.deviceInfo.platform, buildConfig, this.$projectData);
                        return pathToBuildResult;
                    })
                };
                return info;
            });
            const liveSyncInfo = {
                projectDir: this.$projectData.projectDir,
                skipWatcher: !this.$options.watch || this.$options.justlaunch,
                watchAllFiles: this.$options.syncAllFiles
            };
            yield this.$debugLiveSyncService.liveSync(deviceDescriptors, liveSyncInfo);
        });
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.$platformService.isPlatformSupportedForOS(this.platform, this.$projectData)) {
                this.$errors.fail(`Applications for platform ${this.platform} can not be built on this OS`);
            }
            const platformData = this.$platformsData.getPlatformData(this.platform, this.$projectData);
            const platformProjectService = platformData.platformProjectService;
            yield platformProjectService.validate(this.$projectData);
            yield this.$devicesService.initialize({
                platform: this.platform,
                deviceId: this.$options.device,
                emulator: this.$options.emulator,
                skipDeviceDetectionInterval: true,
                skipInferPlatform: true
            });
            if (this.$options.emulator || this.$devicesService.deviceCount === 0) {
                return true;
            }
            if (this.$devicesService.deviceCount > 1) {
                this.$options.emulator = true;
                this.$logger.warn("Multiple devices found! Starting debugger on emulator. If you want to debug on specific device please select device with --device option.".yellow.bold);
            }
            return true;
        });
    }
}
exports.DebugPlatformCommand = DebugPlatformCommand;
class DebugIOSCommand extends DebugPlatformCommand {
    constructor($errors, $devicePlatformsConstants, $logger, $iOSDebugService, $devicesService, $config, $debugDataService, $platformService, $options, $projectData, $platformsData, $iosDeviceOperations, $debugLiveSyncService) {
        super($iOSDebugService, $devicesService, $debugDataService, $platformService, $projectData, $options, $platformsData, $logger, $errors, $debugLiveSyncService, $config);
        this.$errors = $errors;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.platform = this.$devicePlatformsConstants.iOS;
        $iosDeviceOperations.setShouldDispose(false);
    }
    canExecute(args) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.$platformService.isPlatformSupportedForOS(this.$devicePlatformsConstants.iOS, this.$projectData)) {
                this.$errors.fail(`Applications for platform ${this.$devicePlatformsConstants.iOS} can not be built on this OS`);
            }
            return (yield _super("canExecute").call(this, args)) && (yield this.$platformService.validateOptions(this.$options.provision, this.$projectData, this.$platformsData.availablePlatforms.iOS));
        });
    }
}
exports.DebugIOSCommand = DebugIOSCommand;
$injector.registerCommand("debug|ios", DebugIOSCommand);
class DebugAndroidCommand extends DebugPlatformCommand {
    constructor($errors, $devicePlatformsConstants, $logger, $androidDebugService, $devicesService, $config, $debugDataService, $platformService, $options, $projectData, $platformsData, $debugLiveSyncService) {
        super($androidDebugService, $devicesService, $debugDataService, $platformService, $projectData, $options, $platformsData, $logger, $errors, $debugLiveSyncService, $config);
        this.$errors = $errors;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.platform = this.$devicePlatformsConstants.Android;
    }
    canExecute(args) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            return (yield _super("canExecute").call(this, args)) && (yield this.$platformService.validateOptions(this.$options.provision, this.$projectData, this.$platformsData.availablePlatforms.Android));
        });
    }
}
exports.DebugAndroidCommand = DebugAndroidCommand;
$injector.registerCommand("debug|android", DebugAndroidCommand);
